	OPTION EXPLICIT
	OPTION DEFAULT NONE
	CPU 48
	GOTO TestSuite


'************************************************************************************* 
' 
' Peter Mather
' 2.0 		2015-07-02 Modified by Peter Mather
'            Converted to be compatible with MMII 4.7
'
' Graphics library with drivers for SSD1351 display
' SPI output from Micromite MX170 - flexible pinout for control signals

' drawing primitives assume a graphics display where top left is point 0, 0
' function calls are as compatible as possible with the in-built graphics routines as documented for version 4.7
	
' Define which pins are used to interface with the display
' Call S1351.PinDefs BEFORE calling anything else in S1351
' Sub S1351.PinDefs(DC,CS,RST)
'	    DC Pin used for DC
'     CS Pin used for CS
'     RST Pin used to generate Reset signal to display controller
'
'initialise display:
'SUB S1351.init(displayheight)
'     displayheight can be 96 or 128 (pixels)
'
'clear screen:
'SUB S1351.CLS(colour)
'    clear the screen to the colour specified
'    if the colour is omitted the screen is cleared to black
'
'set screen orientation:
'SUB S1351.ROTATE(orientation)
'    valid values are PORTRAIT:0, RPORTRAIT:1, LANDSCAPE:2, RLANDSCAPE:3
'
'plot PIXEL:
'SUB S1351.PIXEL(x, y, c)
'    x, y coordinates of PIXEL, colour
'
'draw circle:
'SUB S1351.CIRCLE(x, y, r, c, f)
'    x, y coordinates of centre, radius, colour of outline, colour to fill
'    if colour to fill is omitted an outline only is drawn
'    to fill with black use the special function S1351.BLACK() as the colour
'    to generate a circle in a single solid colour use the same colour for the outline and the fill
'
'draw triangle:
'SUB S1351.TRIANGLE(x0, y0, x1, y1, 	x2, y2, c, f)
'    three coordinate pairs, colour of outline, colour to fill
'    if colour to fill is omitted an outline only is drawn
'    to fill with black use the special function S1351.BLACK() as the colour
'    to generate a triangle in a single solid colour use the same colour for the outline and the fill
'
'print text:
'SUB S1351.TEXT(x, y, text$, font, size, forecolour, backcolour)
'    x, y coordinates of top left of first character, font number, text string, size, c,  foreground colour,  background colour
'    if the background colour is omitted black will be used
'
'print bitmap:
'SUB S1351.BITMAP(x, y, bitmap, width, height, scale, forecolour, backcolour)
'    x, y coordinates of top left of the bitmap, bitmap, size, character orientation,  foreground colour,  background colour
'    the bitmap must be specifed as an array of integers
'    to output bitmap contained within a single integer use array size 1 and allocate the bimap to element 0
'
'draw line:
'SUB S1351.LINE(x0, y0, x1, y1, colour)
'    x, y of start line, x, y of end line, foreground colour,  background colour
'
'draw rectangle:
'SUB S1351.BOX(x, y, w, h, c, f)
'    x, y coordinate of top left, width, height, colour of outline, colour to fill
'    if colour to fill is omitted an outline only is drawn
'    to fill with black use the special function S1351.BLACK() as the colour
'    to generate a rectangle in a single solid colour use the same colour for the outline and the fill
'
'draw rounded rectangle:
'SUB S1351.RBOX(x, y, w, h, r, c, f)
'    x, y coordinates of top left, width, height, radius of corner, colour of outline, colour to fill
'    if colour to fill is omitted an outline only is drawn
'    to fill with black use the special function S1351.BLACK() as the colour
'    to generate a rounded rectangle in a single solid colour use the same colour for the outline and the fill
'
'scroll region:
'SUB S1351.SCROLL(x ,y, w, h)
'    x, y coordinate of top left, width, height of scroll area
'
' screen horizontal resolution in default orientation:  S1351.MM.HRES.DEFAULT.DEFAULT
' screen vertical resolution in default orientation:  S1351.MM.VRES.DEFAULT.DEFAULT
' screen horizontal resolution in current orientation:  S1351.MM.HRES.DEFAULT
' screen vertical resolution in current orientation:  S1351.MM.VRES.DEFAULT
'

'
sub S1351.init(S1351.displayheight AS INTEGER) 
  if S1351.displayheight=0 then S1351.displayheight=128
	'Command codes for S1351 CFunction
	DIM INTEGER S1351.pix=0, S1351.ffr=1, S1351.filltri=2, S1351.drawline=3, S1351.wc=4, S1351.wd=5
	DIM INTEGER S1351.fpstring=7, S1351.fcirch=8, S1351.circh=9, S1351.Bits=10, S1351.scrolled=11, S1351.initialise=12

	'Orientation codes
	DIM INTEGER LANDSCAPE=2, RLANDSCAPE=3, PORTRAIT=0, RPORTRAIT=1
	
	DIM INTEGER S1351.MM.HRES.DEFAULT, S1351.MM.VRES.DEFAULT, S1351.MM.HRES, S1351.MM.VRES
	S1351.MM.HRES.DEFAULT=128: S1351.MM.VRES.DEFAULT=S1351.displayheight
	S1351.VAR(S1351.wi)=S1351.MM.HRES.DEFAULT :  S1351.VAR(S1351.ht)=S1351.MM.VRES.DEFAULT
'	S1351.VAR(S1351.PORTS)=S1351(S1351.initialise,S1351.VAR())

  const SSD1351_CMD_SETCOLUMN 	  	=&H15
  const SSD1351_CMD_SETROW    		  =&H75
  const SSD1351_CMD_WRITERAM   		  =&H5C
  const SSD1351_CMD_READRAM   		  =&H5D
  dim integer SSD1351_CMD_SETREMAP 	=&HA0
  dim integer SSD1351_CMD_STARTLINE =&HA1
  const SSD1351_CMD_DISPLAYOFFSET 	=&HA2
  const SSD1351_CMD_DISPLAYALLOFF 	=&HA4
  const SSD1351_CMD_DISPLAYALLON  	=&HA5
  const SSD1351_CMD_NORMALDISPLAY 	=&HA6
  const SSD1351_CMD_INVERTDISPLAY 	=&HA7
  const SSD1351_CMD_FUNCTIONSELECT 	=&HAB
  const SSD1351_CMD_DISPLAYOFF 		  =&HAE
  const SSD1351_CMD_DISPLAYON     	=&HAF
  const SSD1351_CMD_PRECHARGE 		  =&HB1
  const SSD1351_CMD_DISPLAYENHANCE	=&HB2
  const SSD1351_CMD_CLOCKDIV 		    =&HB3
  const SSD1351_CMD_SETVSL 		      =&HB4
  const SSD1351_CMD_SETGPIO 		    =&HB5
  const SSD1351_CMD_PRECHARGE2 		  =&HB6
  const SSD1351_CMD_SETGRAY 		    =&HB8
  const SSD1351_CMD_USELUT 		      =&HB9
  const SSD1351_CMD_PRECHARGELEVEL 	=&HBB
  const SSD1351_CMD_VCOMH 		      =&HBE
  const SSD1351_CMD_CONTRASTABC		  =&HC1
  const SSD1351_CMD_CONTRASTMASTER	=&HC7
  const SSD1351_CMD_MUXRATIO        =&HCA
  const SSD1351_CMD_COMMANDLOCK     =&HFD
  const SSD1351_CMD_HORIZSCROLL     =&H96
  const SSD1351_CMD_STOPSCROLL      =&H9E
  const SSD1351_CMD_STARTSCROLL     =&H9F
'
'Reset the display controller
  PIN(S1351.VAR(S1351.RST_I))=1:  SETPIN S1351.VAR(S1351.RST_I),DOUT
  PAUSE 100
  PIN(S1351.VAR(S1351.RST_I))=0
  PAUSE 100
  PIN(S1351.VAR(S1351.RST_I))=1
  PAUSE 100
' Initialization Sequence
  S1351.CFG(SSD1351_CMD_COMMANDLOCK,1,&H12)  
  S1351.CFG(SSD1351_CMD_COMMANDLOCK,1,&HB1)
  S1351.CFG(SSD1351_CMD_DISPLAYOFF)
  S1351.CFG(SSD1351_CMD_CLOCKDIV,1,&HF1)
  S1351.CFG(SSD1351_CMD_MUXRATIO,1,127)
  S1351.CFG(SSD1351_CMD_SETREMAP,1,&H74)
  S1351.CFG(SSD1351_CMD_SETCOLUMN,2,&H00,&H7F)
  S1351.CFG(SSD1351_CMD_SETROW,2,&H00,&H7F)
'  if (S1351.displayheight = 96) THEN
'  S1351.CFG(SSD1351_CMD_STARTLINE,1,96)
'  else 
'  S1351.CFG(SSD1351_CMD_STARTLINE,0,0)
'  endif
  S1351.CFG(SSD1351_CMD_DISPLAYOFFSET,1,&H0)
  S1351.CFG(SSD1351_CMD_SETGPIO,1,&H00)
  S1351.CFG(SSD1351_CMD_FUNCTIONSELECT,1,&H01)
  S1351.CFG(SSD1351_CMD_PRECHARGE,1,&H32)
  S1351.CFG(SSD1351_CMD_VCOMH,1,&H05)
  S1351.CFG(SSD1351_CMD_NORMALDISPLAY)
  S1351.CFG(SSD1351_CMD_CONTRASTABC,3,&HC8,&H80,&HC8)
  S1351.CFG(SSD1351_CMD_CONTRASTMASTER,1,&H0F)
  S1351.CFG(SSD1351_CMD_SETVSL ,3,&HA0,&HB5,&H55)
  S1351.CFG(SSD1351_CMD_PRECHARGE2,1,&H01)
  S1351.CFG(SSD1351_CMD_DISPLAYON)
  S1351.CFG(SSD1351_CMD_WRITERAM)  
  S1351.rotate(PORTRAIT)
  S1351.cls(S1351.black())
end sub
' Set up the pin definitions
' Call S1351.PinDefs BEFORE calling anything else in S1351 
' DC, any valid pin
' CS, any valid pin
' RST, any valid pin
'
SUB S1351.PinDefs(DC%,CS%,RST%)

	'Holds Parameters for use in the Cfunction
	DIM INTEGER S1351.VAR(10) 
	
	'Subscripts in S1351.VAR for each parameter
	DIM INTEGER S1351.Default_Rotation=0 'default screen orientation for this display type, in this case LANDSCAPE
	DIM INTEGER S1351.wi=0,S1351.ht=1,S1351.rot=2,S1351.nr=3,S1351.X=4,S1351.Y=5,S1351.seed=6  

	
	'Subscripts for the control signals
	DIM INTEGER S1351.DC_I=7, S1351.CS_I=8, S1351.RST_I=9,	DIM INTEGER S1351.PORTS=10
	pin(RST%)=1:  SETPIN RST%,dout	
	pin(DC%)=1:  SETPIN DC%,dout	
	pin(CS%)=1:  SETPIN CS%,dout	
	S1351.VAR(S1351.RST_I)=RST% : 	S1351.VAR(S1351.DC_I)=DC%   :  S1351.VAR(S1351.CS_I)=CS%  
  spi open 10000000,3,8 'run at maximum speed
  poke word &HBF805830,1 'frig for 4.7b8
	
END SUB
'
SUB S1351.CFG(c%, n AS INTEGER, d1%, d2%, d3%, d4%, d5%, d6%, d7%, d8%)
LOCAL INTEGER i
   i=S1351(S1351.wc, c%)
   IF n>0 THEN i=S1351(S1351.wd, d1%)
   IF n>1 THEN i=S1351(S1351.wd, d2%)
   IF n>2 THEN i=S1351(S1351.wd, d3%)

END SUB
'
sub S1351.ROTATE(x%) 'sets screen rotation: 0=PORTRAIT, 1=RPORTRAIT, 2=LANDSCAPE, 3-RLANDSCAPE
    S1351.VAR(s1351.rot)=(x% and 3)
    select case x%
        case portrait
		     S1351.MM.VRES=S1351.MM.VRES.DEFAULT
		     S1351.MM.HRES=s1351.MM.HRES.DEFAULT
         case rportrait
		       S1351.MM.VRES=S1351.MM.VRES.DEFAULT
		       S1351.MM.HRES=S1351.MM.HRES.DEFAULT
         case landscape
		       S1351.MM.VRES=S1351.MM.HRES.DEFAULT
		       S1351.MM.HRES=S1351.MM.VRES.DEFAULT
         endif
          case rlandscape
		       S1351.MM.VRES=S1351.MM.HRES.DEFAULT
		       S1351.MM.HRES=S1351.MM.VRES.DEFAULT
    end select
end sub
'
' Generic drawing routines
'
sub S1351.CLS(scol as integer)
  local integer i=S1351.VAR(S1351.rot)
  S1351.VAR(S1351.rot)=PORTRAIT
  local integer j=S1351(S1351.ffr,scol, 0,0,S1351.VAR(S1351.wi),S1351.VAR(S1351.ht), S1351.VAR())
  S1351.VAR(S1351.rot)=i
end sub
'
SUB S1351.CIRCLE(x%, y%, r%, c%, f%)
  LOCAL i%
  IF(f%) THEN 
    i%=S1351(S1351.ffr, f%, x%, y%-r%, 1, 2*r%+1,  S1351.VAR())
    i%=S1351(S1351.fcircH, x%, y%, r%, 3, 0, f%, S1351.VAR())
  ENDIF
  IF(f%<>c%) THEN i%=S1351(S1351.circh, c%, x%, y%, r%, 31, S1351.VAR())
END SUB
 '
FUNCTION S1351.black()AS INTEGER
   S1351.black=&H8000000000000000
END FUNCTION
'
SUB S1351.PIXEL(x AS INTEGER, y AS INTEGER,  c AS INTEGER) 
  LOCAL i%=S1351(S1351.pix, c, x, y, S1351.VAR())
END SUB
 '
SUB S1351.TRIANGLE(x0 AS INTEGER, y0 AS INTEGER, x1 AS INTEGER, y1 AS INTEGER, x2 AS INTEGER, y2 AS INTEGER, c AS INTEGER, f AS INTEGER)
  LOCAL i%
  IF(f) THEN i%=S1351(S1351.filltri, 1,x0,y0,x1,y1,x2,y2,c,S1351.VAR())
  IF(f<>c) THEN
     S1351.LINE(x0, y0, x1, y1, c)
     S1351.LINE(x1, y1, x2, y2, c)
     S1351.LINE(x2, y2, x0, y0, c)
  ENDIF
END SUB
'
SUB S1351.TEXT(x%, y%, text$, font%, sz%, c%, f%) 
  LOCAL j%=font% AND 1
  LOCAL INTEGER i=S1351(S1351.fpstring,j%,x%,y%,sz%,c%,f%,S1351.VAR(),text$)
END SUB
'
SUB S1351.LINE(x0%, y0%, x1%, y1%, c%)
   LOCAL i%=S1351(S1351.drawline, c%, x0%, y0%, x1%, y1%,  S1351.VAR())
END SUB
'
SUB S1351.BOX(x AS INTEGER, y AS INTEGER, w AS INTEGER, h AS INTEGER, c AS INTEGER, f AS INTEGER)
  LOCAL INTEGER a, i
  IF(f) THEN i=S1351(S1351.ffr, f, x, y, w, h,  S1351.VAR()) 
  IF(f<>c) THEN
     i=S1351(S1351.ffr, c, x, y, w, 1,  S1351.VAR()) 
     a=y+h-1
     i=S1351(S1351.ffr, c, x, a, w, 1,  S1351.VAR()) 
     i=S1351(S1351.ffr, c, x, y, 1, h,  S1351.VAR()) 
     a=x+w-1
     i=S1351(S1351.ffr, c, a, y, 1, h,  S1351.VAR()) 
  ENDIF
END SUB
'
SUB S1351.BITMAP(x%, y%, array%(), w%, h%, sz%, c%, f%)
  LOCAL i%=S1351(S1351.Bits, x%,y%,w%,h%,sz%,c%,f%,array%(),S1351.VAR())  
END SUB
'
SUB S1351.RBOX(x AS INTEGER, y AS INTEGER, w AS INTEGER, h AS INTEGER, r AS INTEGER, c AS INTEGER, f AS INTEGER)
  LOCAL INTEGER i
  IF(f) THEN
	  i=S1351(S1351.ffr, f, x+r, y, w-2*r, h,  S1351.VAR()) 
	  i=S1351(S1351.fcircH,x+w-r-1, y+r, r, 1, h-2*r-1, f, S1351.VAR())
	  i=S1351(S1351.fcircH,x+r, y+r, r, 2, h-2*r-1, f, S1351.VAR())
  ENDIF
  IF(f<>c) THEN
	  i=S1351(S1351.ffr, c, x+r, y, w-2*r, 1,  S1351.VAR()) 
	  i=S1351(S1351.ffr, c, x+r, y+h-1, w-2*r, 1,  S1351.VAR()) 
	  i=S1351(S1351.ffr, c, x, y+r, 1, h-2*r,  S1351.VAR()) 
	  i=S1351(S1351.ffr, c, x+w-1, y+r, 1, h-2*r,  S1351.VAR()) 
	  i=S1351(S1351.circh, c, x+r, y+r, r, 1,S1351.VAR())
	  i=S1351(S1351.circh,  c, x+w-r-1, y+r, r, 2,S1351.VAR())
	  i=S1351(S1351.circh,  c, x+w-r-1, y+h-r-1, r, 4,S1351.VAR())
	  i=S1351(S1351.circh,  c, x+r, y+h-r-1, r, 8,S1351.VAR())
  ENDIF
END SUB
'
CFunction S1351
	000004A1
	'writeCommand
	34038000 3c02bf88 ac436134 3c02bf80 ac445820 3c03bf80 8c625810 30420080 
	1040fffd 3c02bf80 8c425820 03e00008 00000000 

	'writeData
	34038000 3c02bf88 ac436138 3c02bf80 ac445820 3c03bf80 8c625810 30420080 
	1040fffd 3c02bf80 8c425820 03e00008 00000000 

	'defineregion
	27bdffd0 afbf002c afb50028 afb40024 afb30020 afb2001c afb10018 afb00014 
	00809021 00a08821 8fa30040 8c620010 8c700008 2494ffff 0286a021 24b3ffff 
	14400015 02679821 240400a1 00002821 0411FFD1 00000000 24020060 16020008 
	00002021 24040060 00002821 0411FFD7 00000000 0000a821 10000033 24100074 
	00002821 0411FFD1 00000000 0000a821 1000002d 24100074 24030003 1443000c 
	24030002 240400a1 00002821 0411FFBA 00000000 00002021 00002821 0411FFC3 
	00000000 24150001 1000001f 24100064 14430014 240400a1 00002821 0411FFAE 
	00000000 24020060 16020008 00002021 24040060 00002821 0411FFB4 00000000 
	24150001 10000010 24100076 00002821 0411FFAE 00000000 24150001 1000000a 
	24100076 00002821 0411FF9B 00000000 00002021 00002821 0411FFA4 00000000 
	0000a821 24100066 36020001 0055800b 240400a0 00002821 0411FF8F 00000000 
	02002021 00102fc3 0411FF98 00000000 12a00026 24040015 24040075 00002821 
	0411FF85 00000000 02402021 00122fc3 0411FF8E 00000000 02802021 00142fc3 
	0411FF8A 00000000 24040015 00002821 0411FF79 00000000 02202021 00112fc3 
	0411FF82 00000000 02602021 00132fc3 0411FF7E 00000000 2404005c 00002821 
	0411FF6D 00000000 8fbf002c 8fb50028 8fb40024 8fb30020 8fb2001c 8fb10018 
	8fb00014 03e00008 27bd0030 00002821 0411FF61 00000000 02402021 00122fc3 
	0411FF6A 00000000 02802021 00142fc3 0411FF66 00000000 24040075 1000ffdc 
	00002821 

	'localfastfillrect
	27bdffd8 afbf0024 afb10020 afb0001c 00808021 8fa20038 70478802 8fa3003c 
	afa30010 00a02021 00c02821 00e03021 00403821 0411FF61 00000000 7e0520c0 
	00101203 3042f800 00a22825 00108143 321007e0 00b02825 00052202 30a500ff 
	34038000 3c02bf88 ac436138 12200010 2631ffff 3c02bf80 ac445820 8c435810 
	30630080 1060fffd 00000000 8c435820 ac455820 8c435810 30630080 1060fffd 
	00000000 8c435820 1620fff3 2631ffff 8fbf0024 8fb10020 8fb0001c 03e00008 
	27bd0028 

	'pixel
	27bdffe0 afbf001c afb00018 00808021 afa70010 00a02021 00c02821 24060001 
	24070001 0411FF34 00000000 7e0320c0 00101203 3042f800 00621025 00108143 
	321007e0 00508025 00101a02 321000ff 34048000 3c02bf88 ac446138 3c02bf80 
	ac435820 3c03bf80 8c625810 30420080 1040fffd 3c02bf80 8c435820 ac505820 
	3c03bf80 8c625810 30420080 1040fffd 3c02bf80 8c425820 8fbf001c 8fb00018 
	03e00008 27bd0020 

	'swap
	8c820000 8ca30000 ac830000 03e00008 aca20000 

	'drawline
	27bdffc0 afbf003c afb60038 afb50034 afb40030 afb3002c afb20028 afb10024 
	afb00020 00809021 8fa40050 8fa20058 8fa30060 8fb40068 afa60010 afa40014 
	afa20018 afa3001c 0083882a 00642823 00831823 0071280a 00c2202a 00461823 
	00c21023 0064100b 0045882a 1220000a 8fa20018 27a40010 27a50014 0411FFDB 
	00000000 27a40018 27a5001c 0411FFD7 00000000 8fa20018 8fa30010 0043102a 
	1040000a 8fa20018 27a40010 27a50018 0411FFCE 00000000 27a40014 27a5001c 
	0411FFCA 00000000 8fa20018 8fa60010 8fa3001c 8fb30014 0263202a 10800007 
	0046a823 00739823 001587c2 02158021 00108043 10000006 24160001 02639823 
	001587c2 02158021 00108043 2416ffff 0046102a 1440001c 8fbf003c 12200007 
	02402021 8fa50014 02803821 0411FF85 00000000 10000007 02138023 00c02821 
	8fa60014 02803821 0411FF7E 00000000 02138023 06010006 8fa60010 8fa20014 
	00561021 afa20014 02158021 8fa60010 24c60001 afa60010 8fa20018 0046102a 
	1040ffe6 8fbf003c 8fb60038 8fb50034 8fb40030 8fb3002c 8fb20028 8fb10024 
	8fb00020 03e00008 27bd0040 

	'DBitmap
	27bdffa8 afbf0054 afbe0050 afb7004c afb60048 afb50044 afb40040 afb3003c 
	afb20038 afb10034 afb00030 0080f021 afa5005c 00c09021 afa70064 8fb00068 
	8fa4006c 8fa30070 8fa20078 8c450010 2ca50002 10a00005 8fb30074 8c510000 
	8c450008 10000004 afa50018 8c510008 8c450000 afa50018 7c9620c0 00042a03 
	30a5f800 02c5b025 00042143 308407e0 02c4b025 7c7420c0 00032203 3084f800 
	0284a025 00031943 306307e0 0283a025 0016ba02 32d600ff 0014aa02 329400ff 
	afa20010 03c02021 8fa5005c 72123002 8fa20064 72023802 0411FE6D 00000000 
	34038000 3c02bf88 ac436138 8fa30064 18600066 02007021 8fa5005c afa5001c 
	00121023 afa20024 7072c802 2739ffff afb20028 00006821 afa00020 240f0001 
	10000056 3c04bf80 04400025 0051282b 50a00024 24630001 05020022 24630001 
	51400020 24630001 91850000 00ab2824 10a0000f 00000000 ac975820 8c855810 
	30a50080 10a0fffd 00000000 8c855820 ac965820 8c855810 30a50080 10a0fffd 
	00000000 8c855820 1000000e 24630001 ac955820 8c855810 30a50080 10a0fffd 
	00000000 8c855820 ac945820 8c855810 30a50080 10a0fffd 00000000 8c855820 
	24630001 1470ffd8 24420001 24c60001 24e7ffff 10d20010 012e4821 00cd1021 
	24430007 284c0000 006c100b 000260c3 026c6021 000717c3 00021742 00e25821 
	316b0007 01625823 016f5804 01201021 1000ffc5 00001821 27180001 17100004 
	25080001 10000009 8fa30020 0000c021 1a40fff9 03c04821 03203821 00003021 
	8fa20018 1000ffe5 0102502b 24630001 afa30020 8fa5001c 00ae2821 afa5001c 
	8fa20024 0322c821 8fa30028 8fa50020 8fa20064 10a20005 01a36821 1e00ffeb 
	8fa8001c 1000fff1 8fa30020 8fbf0054 8fbe0050 8fb7004c 8fb60048 8fb50044 
	8fb40040 8fb3003c 8fb20038 8fb10034 8fb00030 03e00008 27bd0058 

	'drawchar
	27bdffd0 afbf002c 8faa0040 8fa90044 8fa20048 8fab004c 7c042420 3c039d00 
	8c630054 00052880 00a31821 8c650000 90a80001 90ac0002 008c682a 15a00019 
	90a30000 90ad0003 01ac6821 008d682a 51a00015 71024002 afa20010 afa60014 
	afa70018 71031002 000210c3 008c6023 704c2002 24840004 00a42821 afa5001c 
	afab0020 01402021 01202821 00603021 01003821 0411FF2B 00000000 1000000b 
	8fbf002c 71024002 afa80010 afab0014 00e02021 01402821 01203021 70623802 
	0411FE55 00000000 8fbf002c 03e00008 27bd0030 

	'testchar
	8fa30010 8c680010 8c690000 70e61002 00442021 2d020002 10400009 8c630008 
	04800012 00001021 04a00010 0124302a 14c0000e 0065302a 14c0000c 00000000 
	2508fffe 2d080002 11000008 24020001 04800006 00001021 04a00004 0064182a 
	14600002 0125282a 38a20001 03e00008 00000000 

	'pstring
	27bdffb8 afbf0044 afbe0040 afb7003c afb60038 afb50034 afb40030 afb3002c 
	afb20028 afb10024 afb00020 0080b821 00a08021 00c09821 00e0a021 8fb50064 
	3c029d00 8c420054 00041880 00621021 8c420000 90560000 92a20000 1840001b 
	8fb20060 72c7f002 24110001 afb20010 02002021 02602821 02803021 02c03821 
	0411FFC2 00000000 10400016 02b11021 80440000 afb00010 afb30014 afb40018 
	afb2001c 02e02821 8fa60058 8fa7005c 0411FF81 00000000 26310001 92a20000 
	0051102a 1040ffe9 021e8021 ae500020 001087c3 ae500024 ae530028 00139fc3 
	ae53002c 8fbf0044 8fbe0040 8fb7003c 8fb60038 8fb50034 8fb40030 8fb3002c 
	8fb20028 8fb10024 8fb00020 03e00008 27bd0048 

	'dcirch
	27bdffb8 afbf0044 afbe0040 afb7003c afb60038 afb50034 afb40030 afb3002c 
	afb20028 afb10024 afb00020 0080a821 00a0a021 00c09821 00e08821 8fa30058 
	30620010 10400017 8fb6005c 00e63021 02c03821 0411FE0A 00000000 02a02021 
	02802821 02713023 02c03821 0411FE04 00000000 02a02021 02342821 02603021 
	02c03821 0411FDFE 00000000 02a02021 02912823 02603021 02c03821 0411FDF8 
	00000000 1a200054 24120001 02519023 0011b823 0017b840 00008021 8fa20058 
	305e0004 30430002 afa30014 30420008 afa20018 8fa30058 30630001 afa30010 
	06400004 00000000 2631ffff 26f70002 02579021 13c0000d 26100001 02a02021 
	02142821 02333021 02c03821 0411FDDC 00000000 02a02021 02342821 02133021 
	02c03821 0411FDD6 00000000 8fa20014 1040000e 8fa30018 02a02021 02142821 
	02713023 02c03821 0411FDCD 00000000 02a02021 02342821 02703023 02c03821 
	0411FDC7 00000000 8fa30018 1060000e 8fa20010 02a02021 02912823 02133021 
	02c03821 0411FDBE 00000000 02a02021 02902823 02333021 02c03821 0411FDB8 
	00000000 8fa20010 1040000e 0211102a 02a02021 02912823 02703023 02c03821 
	0411FDAF 00000000 02a02021 02902823 02713023 02c03821 0411FDA9 00000000 
	0211102a 10400004 00101040 24420001 1000ffbb 02429021 8fbf0044 8fbe0040 
	8fb7003c 8fb60038 8fb50034 8fb40030 8fb3002c 8fb20028 8fb10024 8fb00020 
	03e00008 27bd0048 

	'fcirch
	27bdffb0 afbf004c afbe0048 afb70044 afb60040 afb5003c afb40038 afb30034 
	afb20030 afb1002c afb00028 0080b821 00a0b021 8fb50068 18c00048 8fbe0070 
	24100001 02068023 0006a023 0014a040 00c08821 24130003 00009021 30e20001 
	afa20018 8fa30060 24630001 afa30020 30e70002 afa7001c 06000004 8fa20018 
	2631ffff 26940002 02148021 10400016 26520001 00111040 8fa30020 00621021 
	afa20010 afbe0014 02a02021 02572821 02d13023 24070001 0411FD36 00000000 
	8fa30060 02631021 afa20010 afbe0014 02a02021 02372821 02d23023 24070001 
	0411FD2C 00000000 8fa2001c 10400017 0251102a 00111040 8fa30020 00621021 
	afa20010 afbe0014 02a02021 02f22823 02d13023 24070001 0411FD1E 00000000 
	8fa30060 02631021 afa20010 afbe0014 02a02021 02f12823 02d23023 24070001 
	0411FD14 00000000 0251102a 10400003 02138021 1000ffc8 26730002 8fbf004c 
	8fbe0048 8fb70044 8fb60040 8fb5003c 8fb40038 8fb30034 8fb20030 8fb1002c 
	8fb00028 03e00008 27bd0050 

	'ftri
	27bdff90 afbf006c afbe0068 afb70064 afb60060 afb5005c afb40058 afb30054 
	afb20050 afb1004c afb00048 afa40070 188000df 8fb20090 00a0a821 00c0b021 
	00e0b821 8fbe0080 8fa20084 afa20038 8fa30088 afa3003c 8fa4008c afa40040 
	afa00044 8ea20000 afa20020 8ec30000 afa30024 8ee20000 afa20028 8fc20000 
	afa2002c 8fa50038 8ca40000 afa40030 8fa5003c 8ca40000 afa40034 8fa40040 
	0043102a 10400009 8c910000 27a40024 27a5002c 0411FD2F 00000000 27a40020 
	27a50028 0411FD2B 00000000 8fa20034 8fa3002c 0043102a 1040000a 8fa2002c 
	27a40034 27a5002c 0411FD22 00000000 27a40030 27a50028 0411FD1E 00000000 
	8fa2002c 8fa30024 0043102a 1040000a 8fb00024 27a40024 27a5002c 0411FD15 
	00000000 27a40020 27a50028 0411FD11 00000000 8fb00024 8fa20034 16020023 
	8fb3002c 8fa20020 afa20018 afa2001c 8fa30028 0062202a 10800003 0043102a 
	10000003 afa30018 54400001 afa3001c 8fa20030 8fa30018 0043182a 10600003 
	8fa3001c 10000004 afa20018 0062182a 54600001 afa2001c 8fa50018 8fa2001c 
	24420001 00451023 afa20010 afb20014 02202021 8fa60024 24070001 0411FC92 
	00000000 10000069 8fa20044 00531026 0002102b 02629823 0270102a 14400030 
	8fa20034 24140001 8fa20020 8fa30024 02032823 8fa60028 00c23023 70a63002 
	8fa4002c 00832023 00c4001a 008001f4 00002012 00822021 afa40018 8fa60030 
	00c23023 70a62802 8fa60034 00c31823 00a3001a 006001f4 00001812 00621021 
	afa2001c 0044102a 10400006 8fa20018 27a40018 27a5001c 0411FCC6 00000000 
	8fa20018 8fa7001c 24e70001 afb40010 afb20014 02202021 00402821 02003021 
	00e23823 0411FC60 00000000 26100001 0270102a 1040ffd5 8fa20020 8fa20034 
	0050182a 54600031 8fa20044 24130001 8fa60028 8fa40020 8fa50030 8fa8002c 
	02083823 00a61823 70e33802 00484023 00e8001a 010001f4 00001812 00661821 
	afa30018 8fa70024 02073023 00a42823 70c53002 00471023 00c2001a 004001f4 
	00001012 00442021 afa4001c 0083202a 10800006 8fa20018 27a40018 27a5001c 
	0411FC94 00000000 8fa20018 8fa7001c 24e70001 afb30010 afb20014 02202021 
	00402821 02003021 00e23823 0411FC2E 00000000 26100001 8fa20034 0050182a 
	1060ffd4 8fa60028 8fa20044 24420001 afa20044 26b50008 26d60008 26f70008 
	8fa30038 24630008 afa30038 8fa4003c 24840008 afa4003c 8fa50040 24a50008 
	afa50040 8fa30070 1443ff2e 27de0008 8fbf006c 8fbe0068 8fb70064 8fb60060 
	8fb5005c 8fb40058 8fb30054 8fb20050 8fb1004c 8fb00048 03e00008 27bd0070 

	'main
	27bdffc8 afbf0034 afb00030 00808021 8fa80048 8faa004c 8fa90050 8fab0054 
	8fac0058 8c820000 8c830004 00432025 1480000a 8fad005c 8ca40000 8cc50000 
	8ce60000 01003821 0411FC28 00000000 00002021 100000a1 00002821 24040001 
	1444000f 24040002 1460000d 00000000 8ca40000 8cc50000 8ce60000 8d070000 
	8d420000 afa20010 afa90014 0411FBE6 00000000 00002021 10000090 00002821 
	14440011 24040003 1460000f 00000000 8ca40000 afaa0010 afa90014 afab0018 
	afac001c afad0020 00c02821 00e03021 01003821 0411FED2 00000000 00002021 
	1000007e 00002821 1444001b 24040004 14600019 00000000 8ca40000 8ca50004 
	8cc20000 8cc30004 8ce60000 afa60010 000637c3 afa60014 8d060000 afa60018 
	000637c3 afa6001c 8d460000 afa60020 000637c3 afa60024 afa90028 00403021 
	00603821 0411FC18 00000000 8e040000 10000062 8e050004 1444000a 24040005 
	14600008 00000000 8ca40000 8ca50004 0411FB02 00000000 8e040000 10000057 
	8e050004 1444000a 24040007 14600008 00000000 8ca40000 8ca50004 0411FB04 
	00000000 8e040000 1000004c 8e050004 14440012 24040008 14600010 00000000 
	8ca40000 8cc50000 8ce60000 8d070000 8d420000 afa20010 8d220000 afa20014 
	afab0018 afac001c 0411FD5B 00000000 00002021 10000039 00002821 14440013 
	24040009 14600011 00000000 8ca40000 8cc50000 8ce60000 8d070000 8d420000 
	afa20010 8d220000 8d230004 afa20018 afa3001c afab0020 0411FE16 00000000 
	8e040000 10000025 8e050004 1444000f 2404000a 1460000d 00000000 8ca40000 
	8cc50000 8ce60000 8d070000 8d420000 afa20010 afa90014 0411FD7C 00000000 
	8e040000 10000015 8e050004 14440016 8fbf0034 14600015 8fb00030 8ca40000 
	8cc50000 8ce60000 8d070000 8d420000 afa20010 8d220000 afa20014 8d620000 
	afa20018 afac001c afad0020 0411FC21 00000000 00002021 00002821 00801021 
	00a01821 8fbf0034 8fb00030 03e00008 27bd0038 
End CFunction   'MIPS32 M4K
'
'----------------------------------------------------------------------------
	'Cut here and throw away below when using the Library in your own program
testsuite:
'
S1351.PinDefs(15,11,21) 
S1351.init(96)
do
  S1351.test
loop
end
'
sub S1351.test
  local integer i,j, xfs, yfs, x0, y0, x1, y1, wi, ht, sz, bitmap(1)=(&HFFC3A191898583FF,0)
  local float tn
  for j=0 to 3
    S1351.cls(0)
    S1351.rotate(j)
    S1351.BOX(0, 0,S1351.MM.HRES,S1351.MM.VRES,rgb(red))
    FOR i=10 TO 0 STEP -1
      S1351.TEXT(S1351.MM.HRES/2-32, S1351.MM.VRES/2-25, STR$(i,2) ,1 ,1, rgb(green))
      PAUSE 500
    NEXT i
  next j
  S1351.cls(0)
  S1351.TEXT(S1351.MM.HRES/2-48, 23, "  " ,1 ,1, rgb(green))
  S1351.rotate(PORTRAIT)
  S1351.TEXT(6, 4, "Welcome to the", 0,1 , rgb(green))
  S1351.TEXT(32, 17, "Micromite", 0,1 , rgb(red))
  S1351.TEXT(44,30,"MK2",0,2 ,rgb(red))
  S1351.TEXT(30, 54,  "in colour", 0,1, rgb(yellow))
  S1351.TEXT(38, 67,  "SSD1351", 0,1, rgb(cyan))
  S1351.TEXT(26, 80,  "controller", 0,1, rgb(cyan))
  S1351.BOX(0, 0,S1351.MM.HRES,S1351.MM.VRES,rgb(red))
  pause 4000
' demo fast line draw
  S1351.cls(S1351.black())
  x0=0:y0=0:y1=S1351.VAR(S1351.ht)-1
  for i=0 to 44 step 2
    tn=tan(rad(i))
    x1=S1351.VAR(S1351.wi)*tn
    S1351.line(x0, y0, x1, y1, rgb(red))
  next i
  x1=S1351.VAR(S1351.wi)-1
  for i=0 to 44 step 2
    tn=tan(rad(i))
    y1=S1351.VAR(S1351.ht)*tn
    S1351.line(x0, y0, x1, y1, rgb(red))
  next i
  S1351.BOX(0, 0,S1351.MM.HRES,S1351.MM.VRES,rgb(blue))
  pause 2000
  S1351.cls(0)
'demo fast text write
  FOR i=0 TO S1351.MM.VRES-12 STEP 13
    S1351.TEXT(0, i, "Hello world:Bye world:Hello world:Bye world:Hello world:Bye world:Hello world:Bye world", 0,1, rgb(white))
  NEXT i
  PAUSE 2000
  S1351.ROTATE(RPORTRAIT)
  S1351.CLS()
  FOR i=0 TO S1351.MM.VRES-25 STEP 26
    S1351.TEXT(0, i, "MK2:Invert:MK2:Invert", 0,2, rgb(red))
  NEXT i
  PAUSE 2000
  S1351.ROTATE(LANDSCAPE)
  S1351.CLS()
  FOR i=0 TO S1351.MM.VRES-12 STEP 13
    S1351.TEXT(0,i, "Rotate right:Rotate right:Rotate right", 0,1, rgb(yellow))
  NEXT i
  PAUSE 2000
  S1351.ROTATE(RLANDSCAPE)
  S1351.CLS()
  FOR i=0 TO S1351.MM.VRES-12 STEP 13
    S1351.TEXT(0,i, "Rotate Left:Rotate Left:Rotate Left", 0,1, rgb(blue))
  NEXT i
  PAUSE 2000
'
for i=0 to 3
  S1351.CLS()
  S1351.rotate(i)
  ' test drawing rectangles
  S1351.cls(S1351.black())
     S1351.BOX(4, 4, 32, 37, rgb(blue),rgb(blue))
     S1351.BOX(2, 2, 36, 41, rgb(red))
' test drawing rounded rectangles
     S1351.RBOX(S1351.mm.hres-36, 4, 32, 37, 6, rgb(blue), rgb(blue))
     S1351.RBOX(S1351.mm.hres-38, 2, 36, 41, 6, rgb(red))
' test drawing circles
     S1351.CIRCLE(20, S1351.mm.vres-20, 17, rgb(blue), rgb(blue))
     S1351.CIRCLE(20, S1351.mm.vres-20, 19, rgb(red))
' test drawing triangles and line drawing
     S1351.TRIANGLE(S1351.mm.hres-4, S1351.mm.vres-4, S1351.mm.hres-40, S1351.mm.vres-6, S1351.mm.hres-22, S1351.mm.vres-24, rgb(blue), rgb(blue))
     S1351.TRIANGLE(S1351.mm.hres-1, S1351.mm.vres-2, S1351.mm.hres-44, S1351.mm.vres-4, S1351.mm.hres-22, S1351.mm.vres-27, rgb(red))
  pause 2000
next i
end sub
